@@ -76,10 +76,9 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_GET_TABINFOREQ              24
 #define GSN_GET_TABINFO_CONF            190
 
-/* Signals removed from 8.0.27 */
-/* 683 used to be GET_TABLEID_REQ, no longer used */
-/* 684 used to be GET_TABLEID_REF, no longer used */
-/* 685 used to be GET_TABLEID_CONF, no longer used */
+#define GSN_GET_TABLEID_REQ             683
+#define GSN_GET_TABLEID_REF             684
+#define GSN_GET_TABLEID_CONF            685
 
 #define GSN_DIHNDBTAMPER                25
 #define GSN_NODE_FAILREP                26
@@ -316,9 +315,11 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_DIGETNODESCONF              210
 #define GSN_DIGETNODESREF               211
 #define GSN_DIGETNODESREQ               212
-#define GSN_DIH_SCAN_GET_NODES_REQ      213
-#define GSN_DIH_SCAN_GET_NODES_REF      215
-#define GSN_DIH_SCAN_GET_NODES_CONF     214
+
+/*
+ Signal 213, 214, 215 no longer used, was
+ DIH_SCAN_GET_NODES_REQ/CONF/REF
+*/
 
 #define GSN_DIH_RESTARTCONF             217
 #define GSN_DIH_RESTARTREF              218
@@ -846,9 +847,9 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_CHECK_NODE_RESTARTCONF      619
 
 #define GSN_GET_CPU_USAGE_REQ           620
-#define GSN_621
-#define GSN_622
-#define GSN_623
+#define GSN_OVERLOAD_STATUS_REP         621
+#define GSN_SEND_THREAD_STATUS_REP      622
+#define GSN_NODE_OVERLOAD_STATUS_ORD    623
 
 #define GSN_CREATE_FK_REQ               627
 #define GSN_CREATE_FK_REF               628
@@ -880,6 +881,10 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_648
 #define GSN_649
 
+#define GSN_SET_WAKEUP_THREAD_ORD       657
+#define GSN_WAKEUP_THREAD_ORD           658
+#define GSN_SEND_WAKEUP_THREAD_ORD      659
+
 #define GSN_UTIL_CREATE_LOCK_REQ        132
 #define GSN_UTIL_CREATE_LOCK_REF        133
 #define GSN_UTIL_CREATE_LOCK_CONF       188
@@ -1111,5 +1116,8 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_END_TOREP                   781
 #define GSN_LOCAL_RECOVERY_COMP_REP     782
 
+#define GSN_PROCESSINFO_REP             783
+
 #define GSN_CHECK_LCP_IDLE_ORD          824
+
 #endif
