@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -828,6 +828,7 @@ void getTextInfoEvent(QQQQ) {
 const char bytes_unit[]= "B";
 const char kbytes_unit[]= "KB";
 const char mbytes_unit[]= "MB";
+const char gbytes_unit[]= "GB";
 static void convert_unit(unsigned &data, const char *&unit)
 {
   if (data < 16*1024)
@@ -845,6 +846,27 @@ static void convert_unit(unsigned &data, const char *&unit)
   unit= mbytes_unit;  
 }
 
+static void convert_unit64(Uint64 &data, const char *&unit)
+{
+  if((data >> 32) == 0)
+  {
+    Uint32 data_lo = (Uint32) data;
+    convert_unit(data_lo, unit);
+    data = data_lo;
+    return;
+  }
+  if ((data%(1024*1024*1024)) == 0)
+  {
+    data = data/(1024*1024*1024);
+    unit = gbytes_unit;
+  }
+  else
+  {
+    data = (data+1024*1024-1)/(1024*1024);
+    unit = mbytes_unit;
+  }
+}
+
 void getTextEventBufferStatus(QQQQ) {
   unsigned used= theData[1], alloc= theData[2], max_= theData[3];
   const char *used_unit, *alloc_unit, *max_unit;
@@ -862,6 +884,100 @@ void getTextEventBufferStatus(QQQQ) {
 		       theData[5], theData[4],
 		       theData[7], theData[6]);
 }
+
+
+/** Give the text for the reason enum
+ * ndb_logevent_event_buffer_status_report_reason defined ndb_logevent.h
+ */
+const char *ndb_logevent_eventBuff_status_reasons[] = {
+  "NO_REPORT",
+  "COMPLETELY_BUFFERING",
+  "PARTIALLY_DISCARDING",
+  "COMPLETELY_DISCARDING",
+  "PARTIALLY_BUFFERING",
+  "BUFFERED_EPOCHS_OVER_THRESHOLD",
+  "ENOUGH_FREE_EVENTBUFFER",
+  "LOW_FREE_EVENTBUFFER",
+  "EVENTBUFFER_USAGE_HIGH"
+};
+
+const char* getReason(Uint32 reason)
+{
+  if (reason < NDB_ARRAY_SIZE(ndb_logevent_eventBuff_status_reasons))
+    return ndb_logevent_eventBuff_status_reasons[reason];
+  return "UNKNOWN reason code";
+}
+
+void getTextEventBufferStatus2(QQQQ) {
+  unsigned used= theData[1], alloc= theData[2], max_= theData[3];
+  const char *used_unit, *alloc_unit, *max_unit;
+  convert_unit(used, used_unit);
+  convert_unit(alloc, alloc_unit);
+  convert_unit(max_, max_unit);
+
+  BaseString used_pct_txt;
+  if (alloc != 0)
+  {
+    used_pct_txt.assfmt("(%d%% of alloc)",
+             (Uint32)((((Uint64)theData[1])*100)/theData[2]));
+  }
+
+  BaseString allocd_pct_txt;
+  if (max_ != 0)
+  {
+    allocd_pct_txt.assfmt("(%d%% of max)",
+             (Uint32)((((Uint64)theData[2])*100)/theData[3]));
+  }
+
+  BaseString::snprintf(m_text, m_text_len,
+		       "Event buffer status (0x%x): used=%d%s%s alloc=%d%s%s "
+		       "max=%d%s%s latest_consumed_epoch=%u/%u "
+                       "latest_buffered_epoch=%u/%u "
+                       "report_reason=%s",
+		       theData[8], used, used_unit, used_pct_txt.c_str(),
+		       alloc, alloc_unit, allocd_pct_txt.c_str(),
+		       max_, max_unit, (max_ == 0) ? "(unlimited)" : "",
+		       theData[5], theData[4],
+		       theData[7], theData[6],
+                       getReason(theData[9]));
+}
+
+void getTextEventBufferStatus3(QQQQ) {
+  Uint64 used = ((Uint64)theData[10] << 32) | theData[1];
+  Uint64 alloc = ((Uint64)theData[11] << 32) | theData[2];
+  Uint64 max_ = ((Uint64)theData[12] << 32) | theData[3];
+
+  const char *used_unit, *alloc_unit, *max_unit;
+  convert_unit64(used, used_unit);
+  convert_unit64(alloc, alloc_unit);
+  convert_unit64(max_, max_unit);
+
+  BaseString used_pct_txt;
+  if (alloc != 0)
+  {
+    used_pct_txt.assfmt("(%u%% of alloc)",
+             (Uint32)((used*100)/alloc));
+  }
+
+  BaseString allocd_pct_txt;
+  if (max_ != 0)
+  {
+    allocd_pct_txt.assfmt("(%u%% of max)",
+             (Uint32)((alloc*100)/max_));
+  }
+
+  BaseString::snprintf(m_text, m_text_len,
+		       "Event buffer status (0x%x): used=%llu%s%s alloc=%llu%s%s "
+		       "max=%llu%s%s latest_consumed_epoch=%u/%u "
+                       "latest_buffered_epoch=%u/%u "
+                       "report_reason=%s",
+		       theData[8], used, used_unit, used_pct_txt.c_str(),
+		       alloc, alloc_unit, allocd_pct_txt.c_str(),
+		       max_, max_unit, (max_ == 0) ? "(unlimited)" : "",
+		       theData[5], theData[4],
+		       theData[7], theData[6],
+                       getReason(theData[9]));
+}
 void getTextWarningEvent(QQQQ) {
   BaseString::snprintf(m_text, m_text_len, "%s", (char *)&theData[1]);
 }
@@ -907,14 +1023,28 @@ void getTextBackupFailedToStart(QQQQ) {
 		       refToNode(theData[1]), theData[2]);
 }
 void getTextBackupCompleted(QQQQ) {
+  const Uint32 node_bitmap_size = NdbNodeBitmask::Size;
+  // Build 64-bit data bytes and records by assembling 32-bit signal parts
+  const Uint64 bytes_hi = theData[9 + node_bitmap_size];
+  const Uint64 records_hi = theData[10 + node_bitmap_size];
+  const Uint64 data_bytes = (bytes_hi << 32) | theData[5];
+  const Uint64 data_records = (records_hi << 32) | theData[6];
+
+  // Build 64-bit log bytes and records by assembling 32-bit signal parts
+  const Uint64 bytes_hi_log = theData[11 + node_bitmap_size];
+  const Uint64 records_hi_log = theData[12 + node_bitmap_size];
+  const Uint64 log_bytes = theData[7] | (bytes_hi_log << 32);
+  const Uint64 log_records = theData[8] | (records_hi_log << 32);
+
   BaseString::snprintf(m_text, m_text_len, 
 		       "Backup %u started from node %u completed." 
 		       " StartGCP: %u StopGCP: %u"
-		       " #Records: %u #LogRecords: %u"
-		       " Data: %u bytes Log: %u bytes",
-		       theData[2], refToNode(theData[1]),
-		       theData[3], theData[4], theData[6], theData[8],
-		       theData[5], theData[7]);
+		       " #Records: %llu #LogRecords: %llu"
+		       " Data: %llu bytes Log: %llu bytes",
+                       theData[2], refToNode(theData[1]),
+                       theData[3], theData[4],
+                       data_records, log_records,
+                       data_bytes, log_bytes);
 }
 void getTextBackupStatus(QQQQ) {
   if (theData[1])
@@ -1138,7 +1268,7 @@ void getTextSubscriptionStatus(QQQQ)
   case(1): // SubscriptionStatus::DISCONNECTED
     BaseString::snprintf(m_text, m_text_len,
                          "Disconnecting node %u because it has "
-                         "exceeded MaxBufferedEpochs (%u > %u), epoch %u/%u",
+                         "exceeded MaxBufferedEpochs (%u >= %u), epoch %u/%u",
                          theData[2],
                          theData[5],
                          theData[6],
@@ -1516,6 +1646,8 @@ const EventLoggerBase::EventRepLogLevelMatrix EventLoggerBase::matrix[] = {
   ROW(CreateLogBytes,          LogLevel::llInfo,  11, Logger::LL_INFO ),
   ROW(InfoEvent,               LogLevel::llInfo,   2, Logger::LL_INFO ),
   ROW(EventBufferStatus,       LogLevel::llInfo,   7, Logger::LL_INFO ),
+  ROW(EventBufferStatus2,       LogLevel::llInfo,   7, Logger::LL_INFO ),
+  ROW(EventBufferStatus3,       LogLevel::llInfo,   7, Logger::LL_INFO ),
 
   //Single User
   ROW(SingleUser,              LogLevel::llInfo,   7, Logger::LL_INFO ),
