@@ -1,21 +1,38 @@
-/* Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
 
 #include <mt.hpp>
 #include "LocalProxy.hpp"
 
+#include <signaldata/RouteOrd.hpp>
+
+//#define DBINFO_SCAN_TRACE
+#ifdef DBINFO_SCAN_TRACE
+#include <debugger/DebuggerNames.hpp>
+#endif
+
+#define JAM_FILE_ID 437
+
+
 LocalProxy::LocalProxy(BlockNumber blockNumber, Block_context& ctx) :
   SimulatedBlock(blockNumber, ctx)
 {
@@ -27,6 +44,7 @@ LocalProxy::LocalProxy(BlockNumber blockNumber, Block_context& ctx) :
   for (i = 0; i < MaxWorkers; i++)
     c_worker[i] = 0;
 
+  c_anyWorkerCounter = 0;
   c_typeOfStart = NodeState::ST_ILLEGAL_TYPE;
   c_masterNodeId = ZNIL;
 
@@ -285,7 +303,7 @@ LocalProxy::loadWorkers()
   c_workers = mt_get_instance_count(number());
   for (Uint32 i = 0; i < c_workers; i++)
   {
-    jam();
+    jamNoBlock();
     Uint32 instanceNo = workerInstance(i);
 
     SimulatedBlock* worker = newWorker(instanceNo);
@@ -293,10 +311,46 @@ LocalProxy::loadWorkers()
     ndbrequire(this->getInstance(instanceNo) == worker);
     c_worker[i] = worker;
 
+    if (number() == PGMAN && i == (c_workers - 1))
+    {
+      ((Pgman*)worker)->init_extra_pgman();
+    }
+
     mt_add_thr_map(number(), instanceNo);
   }
 }
 
+void
+LocalProxy::forwardToWorkerIndex(Signal* signal, Uint32 index)
+{
+  jam();
+  /**
+   * We statelessly forward to one of our 
+   * workers, including any sections that 
+   * might be attached.
+   */
+  BlockReference destRef = workerRef(index);
+  SectionHandle sh(this, signal);
+  
+  sendSignal(destRef,
+             signal->header.theVerId_signalNumber,
+             signal,
+             signal->getLength(),
+             JBB,
+             &sh);
+};
+
+void
+LocalProxy::forwardToAnyWorker(Signal* signal)
+{
+  jam();
+
+  /* Won't work for fragmented signals */
+  ndbassert(signal->header.m_fragmentInfo == 0);
+
+  forwardToWorkerIndex(signal, getAnyWorkerIndex());
+}
+
 // GSN_READ_CONFIG_REQ
 
 void
@@ -815,8 +869,19 @@ LocalProxy::execNDB_TAMPER(Signal* signal)
 {
   Ss_NDB_TAMPER& ss = ssSeize<Ss_NDB_TAMPER>();
 
-  ndbrequire(signal->getLength() == 1);
-  ss.m_errorInsert = signal->theData[0];
+  const Uint32 siglen = signal->getLength();
+  if (siglen == 1)
+  {
+    ss.m_errorInsert = signal->theData[0];
+    ss.m_haveErrorInsertExtra = false;
+  }
+  else
+  {
+    ndbrequire(siglen == 2);
+    ss.m_errorInsert = signal->theData[0];
+    ss.m_haveErrorInsertExtra = true;
+    ss.m_errorInsertExtra = signal->theData[1];
+  }
 
   SimulatedBlock::execNDB_TAMPER(signal);
   sendREQ(signal, ss);
@@ -828,9 +893,15 @@ LocalProxy::sendNDB_TAMPER(Signal* signal, Uint32 ssId, SectionHandle* handle)
 {
   Ss_NDB_TAMPER& ss = ssFind<Ss_NDB_TAMPER>(ssId);
 
+  Uint32 siglen = 1;
   signal->theData[0] = ss.m_errorInsert;
+  if (ss.m_haveErrorInsertExtra)
+  {
+    signal->theData[1] = ss.m_errorInsertExtra;
+    siglen ++;
+  }
   sendSignalNoRelease(workerRef(ss.m_worker), GSN_NDB_TAMPER,
-                      signal, 1, JBB, handle);
+                      signal, siglen, JBB, handle);
 }
 
 // GSN_TIME_SIGNAL
@@ -1021,11 +1092,6 @@ LocalProxy::sendDROP_TRIG_IMPL_CONF(Signal* signal, Uint32 ssId)
 
 // GSN_DBINFO_SCANREQ
 
-//#define DBINFO_SCAN_TRACE
-#ifdef DBINFO_SCAN_TRACE
-#include <debugger/DebuggerNames.hpp>
-#endif
-
 static Uint32
 switchRef(Uint32 block, Uint32 instance, Uint32 node)
 {
@@ -1328,14 +1394,69 @@ LocalProxy::execAPI_FAILREQ(Signal* signal)
 }
 
 void
-LocalProxy::sendAPI_FAILREQ(Signal* signal, Uint32 ssId, SectionHandle*)
+LocalProxy::sendAPI_FAILREQ(Signal* signal, Uint32 nodeId, SectionHandle*)
 {
-  Ss_API_FAILREQ& ss = ssFind<Ss_API_FAILREQ>(ssId);
+  Ss_API_FAILREQ& ss = ssFind<Ss_API_FAILREQ>(nodeId);
 
-  signal->theData[0] = ssId;
+  /*
+   * It is a requirement that the API_FAILREQ signal is
+   * received as the last signal from the failed 'nodeId'.
+   * It is produced by QMGR and ROUTEed via the TRPMAN(s)
+   * (in the recv-thread) with the intention that it will
+   * enter the job buffers after any signals received
+   * from the failed API node. (Note that the transporter
+   * *is* closed when API_FAILREQ is created, so any more
+   * signals should not arrive from this API node).
+   *
+   * However, sending the API_FAILREQ via the proxy
+   * will insert it in another job buffer queue than signals
+   * received from the API-node:
+   *
+   *  API-request :       recv/TRPMAN-thread  ---> Block-instance
+   *                                               ^
+   *  Routed-API_FAILREQ: recv/TRPMAN-thread      /
+   *                                     \       /
+   *                                       Proxy
+   *
+   * Depening on the order the queues are processed, the
+   * API_FAILREQ may then be processed by the Block-instance
+   * before a API-request from the failed node - Which is
+   * not what we want.
+   *
+   * Thus we let any proxies receiving a API_FAILREQ, route it
+   * back to the recv/TRPMAN. There it will be inserted in
+   * the same queue as the API-requests, and thus remove the
+   * posibilities for being overtaken:
+   *
+   *  Routed-API_FAILREQ: recv/TRPMAN-thread
+   *                                     \
+   *  re-route to TRPMAN:  -----------<-Proxy (we are here)
+   *                       |
+   *                       v
+   *  API-request+FAIL :   recv/TRPMAN-thread  ---> Block-instance
+   */
+
+  /* API_FAILREQ signal: */
+  signal->theData[0] = nodeId;
   signal->theData[1] = reference();
-  sendSignal(workerRef(ss.m_worker), GSN_API_FAILREQ,
-             signal, 2, JBB);
+
+  Uint32 routedSignalSectionI = RNIL;
+  ndbrequire(appendToSection(routedSignalSectionI,
+                             &signal->theData[0],
+                             2));
+  SectionHandle handle(this, routedSignalSectionI);
+
+  /* RouteOrd data */
+  RouteOrd* routeOrd = (RouteOrd*) signal->getDataPtrSend();
+
+  routeOrd->srcRef = reference();
+  routeOrd->gsn = GSN_API_FAILREQ;
+  routeOrd->from = nodeId;
+  routeOrd->dstRef = workerRef(ss.m_worker);
+  /* ROUTE it through the TRPMAN */
+  sendSignal(TRPMAN_REF, GSN_ROUTE_ORD, signal,
+             RouteOrd::SignalLength,
+             JBB, &handle);
 }
 
 void
