@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -43,6 +43,8 @@
 #include <Array.hpp>
 #include <Mutex.hpp>
 
+#include "../dblqh/Dblqh.hpp"
+
 #define JAM_FILE_ID 474
 
 
@@ -57,13 +59,15 @@ public:
   Backup(Block_context& ctx, Uint32 instanceNumber = 0);
   virtual ~Backup();
   BLOCK_DEFINES(Backup);
-  
+ 
+  class Dblqh* c_lqh;
 protected:
 
   void execSTTOR(Signal* signal);
   void execREAD_CONFIG_REQ(Signal* signal);
   void execDUMP_STATE_ORD(Signal* signal);
   void execREAD_NODESCONF(Signal* signal);
+  void execNODE_START_REP(Signal* signal);
   void execNODE_FAILREP(Signal* signal);
   void execINCL_NODEREQ(Signal* signal);
   void execCONTINUEB(Signal* signal);
@@ -118,7 +122,7 @@ protected:
    * DIH signals
    */
   void execDIH_SCAN_TAB_CONF(Signal* signal);
-  void execDIH_SCAN_GET_NODES_CONF(Signal* signal);
+  void execCHECK_NODE_RESTARTCONF(Signal*);
 
   /**
    * FS signals
@@ -225,6 +229,7 @@ public:
     Uint64 noOfRecords;
 
     Uint32 tableId;
+    Uint32 backupPtrI;
     Uint32 schemaVersion;
     Uint32 tableType;
     Uint32 m_scan_cookie;
@@ -242,6 +247,10 @@ public:
 
     Uint32 nextList;
     union { Uint32 nextPool; Uint32 prevList; };
+    /**
+     * Pointer used by c_tableMap
+     */
+    Uint32 nextMapTable;
   };
   typedef Ptr<Table> TablePtr;
 
@@ -310,8 +319,8 @@ public:
     Backup & backup;
     BlockNumber number() const { return backup.number(); }
     EmulatedJamBuffer *jamBuffer() const { return backup.jamBuffer(); }
-    void progError(int line, int cause, const char * extra) { 
-      backup.progError(line, cause, extra); 
+    void progError(int line, int cause, const char * extra, const char * check) {
+      backup.progError(line, cause, extra, check);
     }
   };
   friend struct OperationRecord;
@@ -345,6 +354,8 @@ public:
     Uint32 errorCode;
     BackupFormat::FileType fileType;
     OperationRecord operation;
+    Uint32 m_sent_words_in_scan_batch;
+    Uint32 m_num_scan_req_on_prioa;
     
     Array<Page32> pages;
     Uint32 nextList;
@@ -404,8 +415,8 @@ public:
     
     BlockNumber number() const { return backup.number(); }
     EmulatedJamBuffer *jamBuffer() const { return backup.jamBuffer(); }
-    void progError(int line, int cause, const char * extra) { 
-      backup.progError(line, cause, extra); 
+    void progError(int line, int cause, const char * extra, const char * check) {
+      backup.progError(line, cause, extra, check);
     }
   private:
     Backup & backup;
@@ -448,6 +459,8 @@ public:
     NDB_TICKS m_prev_report;
 
     Uint32 m_gsn;
+    Uint32 m_lastSignalId;
+    Uint32 m_prioA_scan_batches_to_execute;
     CompoundState slaveState; 
     
     Uint32 clientRef;
@@ -537,13 +550,23 @@ public:
     Backup & backup;
     BlockNumber number() const { return backup.number(); }
     EmulatedJamBuffer *jamBuffer() const { return backup.jamBuffer(); }
-    void progError(int line, int cause, const char * extra) { 
-      backup.progError(line, cause, extra); 
+    void progError(int line, int cause, const char * extra, const char * check) {
+      backup.progError(line, cause, extra, check);
     }
   };
   friend struct BackupRecord;
   typedef Ptr<BackupRecord> BackupRecordPtr;
 
+/**
+ * Number of words needed in buff to start a new scan batch
+ * (Which can directly write a number of rows of max size
+ *  into the buffer)
+ */
+#define BACKUP_MIN_BUFF_WORDS (ZRESERVED_SCAN_BATCH_SIZE *   \
+                               (MAX_TUPLE_SIZE_IN_WORDS +    \
+                                MAX_ATTRIBUTES_IN_TABLE +    \
+                                128))
+
   struct Config {
     Uint32 m_dataBufferSize;
     Uint32 m_logBufferSize;
@@ -551,12 +574,15 @@ public:
     Uint32 m_maxWriteSize;
     Uint32 m_lcp_buffer_size;
     
-    Uint32 m_disk_write_speed_sr;
-    Uint32 m_disk_write_speed;
+    Uint64 m_disk_write_speed_min;
+    Uint64 m_disk_write_speed_max;
+    Uint64 m_disk_write_speed_max_other_node_restart;
+    Uint64 m_disk_write_speed_max_own_restart;
+    Uint32 m_backup_disk_write_pct;
     Uint32 m_disk_synch_size;
     Uint32 m_diskless;
     Uint32 m_o_direct;
-    Uint32   m_compressed_backup;
+    Uint32 m_compressed_backup;
     Uint32 m_compressed_lcp;
   };
   
@@ -564,6 +590,12 @@ public:
    * Variables
    */
   Uint32 * c_startOfPages;
+  /**
+   * Map from tableId to tabPtr.i to speed up findTable
+   * If the same table is mapped to several backups we will
+   * look for the table with the correct backupPtr.
+   */
+  Uint32 * c_tableMap;
   NodeId c_masterNodeId;
   SLList<Node> c_nodes;
   NdbNodeBitmask c_aliveNodes;
@@ -573,17 +605,110 @@ public:
   /*
     Variables that control checkpoint to disk speed
   */
-  Uint32 m_curr_disk_write_speed;
-  Uint32 m_words_written_this_period;
-  Uint32 m_overflow_disk_write;
+  bool m_is_lcp_running;
+  bool m_is_backup_running;
+  bool m_is_any_node_restarting;
+  bool m_node_restart_check_sent;
+  bool m_our_node_started;
+  Uint64 m_curr_disk_write_speed;
+  Uint64 m_words_written_this_period;
+  Uint64 m_overflow_disk_write;
   Uint32 m_reset_delay_used;
   NDB_TICKS m_reset_disk_speed_time;
+
+  /**
+   * We check the use of disk write speed limits every 100 milliseconds. The
+   * speed check parameters is also in words, so this means to get the current
+   * speed in bytes per second we need to multiply with 40.
+   */
   static const int  DISK_SPEED_CHECK_DELAY = 100;
+  static const int CURR_DISK_SPEED_CONVERSION_FACTOR_TO_SECONDS = 40;
   
   Uint64 m_monitor_words_written;
   Uint32 m_periods_passed_in_monitor_period;
   NDB_TICKS m_monitor_snapshot_start;
 
+  /**
+   * A number of statistical variables that keep track of
+   * various events and how often they happen.
+   */
+  Uint64 slowdowns_due_to_io_lag;
+  Uint64 slowdowns_due_to_high_cpu;
+  Uint64 disk_write_speed_set_to_min;
+
+  /**
+   * Variables used to keep stats on disk write speeds for
+   * reporting in checkpoint_speed ndbinfo table.
+   * We keep the last 60 seconds of stats and use this to
+   * calculate various aggregates reported in the ndbinfo
+   * table.
+   *
+   * The idea is that next_disk_write_speed_report specifies
+   * the next entry to fill in a speed report into. The
+   * last_disk_write_speed_report points to the oldest one
+   * that we have written so far. At first we write into
+   * index 0, so in the beginning is last_disk_write_speed_report
+   * equal to 0 and next_disk_write_speed_report is pointing to
+   * the next one to write into. When we write into the last
+   * entry (index = 60) then we have written in all entries and
+   * we move the last forward. After that we will always have
+   * last one ahead of next. Since this means that the next
+   * to write isn't available (although it isn't written yet)
+   * we have 61 entries in the array to cover 60 seconds of
+   * time.
+   */
+#define DISK_WRITE_SPEED_REPORT_SIZE 61
+
+#define MILLIS_IN_A_SECOND 1000
+#define MILLIS_ADJUST_FOR_EARLY_REPORT 20
+  struct DiskWriteSpeedReport
+  {
+    Uint64 backup_lcp_bytes_written;
+    Uint64 redo_bytes_written;
+    Uint64 target_disk_write_speed;
+    Uint64 millis_passed;
+  };
+  DiskWriteSpeedReport disk_write_speed_rep[DISK_WRITE_SPEED_REPORT_SIZE];
+  Uint32 last_disk_write_speed_report;
+  Uint32 next_disk_write_speed_report;
+
+  /**
+   * Methods used in control of checkpoint speed
+   */
+  void handle_overflow(void);
+  void calculate_next_delay(const NDB_TICKS curr_time);
+  void monitor_disk_write_speed(const NDB_TICKS curr_time,
+                                const Uint64 millisPassed);
+  void calculate_current_speed_bounds(Uint64& max_speed, Uint64& min_speed);
+  void adjust_disk_write_speed_down(Uint64 min_speed, int adjust_speed);
+  void adjust_disk_write_speed_up(Uint64 max_speed, int adjust_speed);
+  void calculate_disk_write_speed(Signal *signal);
+  void send_next_reset_disk_speed_counter(Signal *signal);
+
+  void restore_disk_write_speed_numbers(void);
+  void calculate_real_disk_write_speed_parameters(void);
+  Uint64 get_new_speed_val32(Signal *signal);
+  Uint64 get_new_speed_val64(Signal *signal);
+
+  /**
+   * Methods used in ndbinfo reporting of checkpoint speed.
+   */
+  void report_disk_write_speed_report(Uint64 bytes_written_this_period,
+                                      Uint64 millis_passed);
+  Uint32 get_disk_write_speed_record(Uint32 start_index);
+  Uint64 calculate_millis_since_finished(Uint32 start_index);
+  void calculate_disk_write_speed_seconds_back(Uint32 seconds_back,
+                                       Uint64 & millis_passed,
+                                       Uint64 & backup_lcp_bytes_written,
+                                       Uint64 & redo_bytes_written);
+  void calculate_std_disk_write_speed_seconds_back(Uint32 seconds_back,
+                             Uint64 millis_passed_total,
+                             Uint64 backup_lcp_bytes_written_total,
+                             Uint64 redo_bytes_written_total,
+                             Uint64 & std_dev_backup_lcp_in_bytes_per_sec,
+                             Uint64 & std_dev_redo_in_bytes_per_sec);
+
+
   STATIC_CONST(NO_OF_PAGES_META_FILE = 
 	       (2*MAX_WORDS_META_FILE + BACKUP_WORDS_PER_PAGE - 1) / 
 	       BACKUP_WORDS_PER_PAGE);
@@ -602,7 +727,7 @@ public:
   ArrayPool<TriggerRecord> c_triggerPool;
 
   void checkFile(Signal*, BackupFilePtr);
-  void checkScan(Signal*, BackupFilePtr);
+  void checkScan(Signal*, BackupRecordPtr, BackupFilePtr);
   void fragmentCompleted(Signal*, BackupFilePtr);
   
   void backupAllData(Signal* signal, BackupRecordPtr);
@@ -628,6 +753,7 @@ public:
   void backupFragmentRef(Signal * signal, BackupFilePtr filePtr);
 
   void nextFragment(Signal*, BackupRecordPtr);
+  void release_tables(BackupRecordPtr);
   
   void sendCreateTrig(Signal*, BackupRecordPtr ptr, TablePtr tabPtr);
   void createAttributeMask(TablePtr tab, Bitmask<MAXNROFATTRIBUTESINWORDS>&);
@@ -641,6 +767,9 @@ public:
                        FragmentPtr,
                        Uint32 delay);
 
+  void init_scan_prio_level(Signal *signal, BackupRecordPtr ptr);
+  bool check_scan_if_raise_prio(Signal *signal, BackupRecordPtr ptr);
+
   void sendDropTrig(Signal*, BackupRecordPtr ptr);
   void sendDropTrig(Signal* signal, BackupRecordPtr ptr, TablePtr tabPtr);
   void dropTrigReply(Signal*, BackupRecordPtr ptr);
@@ -671,7 +800,9 @@ public:
 
 
   NodeId getMasterNodeId() const { return c_masterNodeId; }
-  bool findTable(const BackupRecordPtr &, TablePtr &, Uint32 tableId) const;
+  bool findTable(const BackupRecordPtr &, TablePtr &, Uint32 tableId);
+  void insertTableMap(TablePtr &, Uint32 backupPtrI, Uint32 tableId);
+  void removeTableMap(TablePtr &, Uint32 backupPtrI, Uint32 tableId);
   bool parseTableDescription(Signal*, BackupRecordPtr ptr, TablePtr, const Uint32*, Uint32);
   
   bool insertFileHeader(BackupFormat::FileType, BackupRecord*, BackupFile*);
@@ -721,6 +852,19 @@ public:
   Uint32 instanceKey(BackupRecordPtr ptr) {
     return ptr.p->is_lcp() ? instance() : UserBackupInstanceKey;
   }
+
+  bool is_backup_worker()
+  {
+    return isNdbMtLqh() ? (instance() == UserBackupInstanceKey) :  true;
+  }
+
+  /**
+   * Ugly shared state to allow different worker instances
+   * to detect that a backup is going, although they are
+   * not participating.
+   * Modified by the instance performing backup
+   */  
+  static bool g_is_backup_running;
 };
 
 inline
