@@ -1,24 +1,30 @@
-/* Copyright (c) 2003, 2005, 2006 MySQL AB
-
+/*
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef RECORDS_HPP
 #define RECORDS_HPP
 
-#include <NdbMain.h>
 #include <NdbOut.hpp>
 #include <ndb_types.h>
 
@@ -33,10 +39,8 @@
 #define ZCOMPLETED_GCI_TYPE 8
 #define ZINVALID_COMMIT_TYPE 9
 
-#define MAX_FILE_DESCRIPTORS 40
-
-#define PAGESIZE 8192
-#define NO_PAGES_IN_MBYTE 32
+#define REDOLOG_PAGESIZE 8192
+#define REDOLOG_PAGES_IN_MBYTE 32
 
 #define COMMITTRANSACTIONRECORDSIZE 9
 #define COMPLETEDGCIRECORDSIZE 2
