@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2012, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2012, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -19,7 +19,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #ifndef DBSPJ_H
@@ -50,6 +50,8 @@ public:
   Dbspj(Block_context& ctx, Uint32 instanceNumber = 0);
   virtual ~Dbspj();
 
+  struct Request;
+  struct TreeNode;
 private:
   BLOCK_DEFINES(Dbspj);
 
@@ -71,8 +73,6 @@ private:
 
   void execDIH_SCAN_TAB_REF(Signal*);
   void execDIH_SCAN_TAB_CONF(Signal*);
-  void execDIH_SCAN_GET_NODES_REF(Signal*);
-  void execDIH_SCAN_GET_NODES_CONF(Signal*);
 
   void execSIGNAL_DROPPED_REP(Signal*);
 
@@ -105,8 +105,6 @@ protected:
   //virtual bool getParam(const char* name, Uint32* count);
 
 public:
-  struct Request;
-  struct TreeNode;
   struct ScanFragHandle;
   typedef DataBuffer2<14, LocalArenaPoolImpl> Correlation_list;
   typedef LocalDataBuffer2<14, LocalArenaPoolImpl> Local_correlation_list;
@@ -138,6 +136,8 @@ public:
       TR_ENABLED      = 1 << 0,
       TR_DROPPING     = 1 << 1,
       TR_PREPARED     = 1 << 2
+      ,TR_READ_BACKUP = (1 << 5)
+      ,TR_FULLY_REPLICATED = (1 << 6)
     };
     Uint8 get_enabled()     const { return (m_flags & TR_ENABLED)      != 0; }
     Uint8 get_dropping()    const { return (m_flags & TR_DROPPING)     != 0; }
@@ -555,10 +555,10 @@ public:
     void (Dbspj::*m_start)(Signal*, Ptr<Request>, Ptr<TreeNode>);
 
     /**
-     * This function is used when getting a TRANSID_AI
+     * This function is called when a waited for signal arrives.
+     * Return 'true' if this completes the wait for this treeNode
      */
-    void (Dbspj::*m_execTRANSID_AI)(Signal*,Ptr<Request>,Ptr<TreeNode>,
-				    const RowPtr&);
+    bool (Dbspj::*m_countSignal)(const Signal*,Ptr<Request>,Ptr<TreeNode>);
 
     /**
      * This function is used when getting a LQHKEYREF
@@ -679,18 +679,21 @@ public:
       SFH_WAIT_CLOSE   = 4
     };
 
-    void init(Uint32 fid) {
+    void init(Uint32 fid, bool readBackup)
+    {
       m_ref = 0;
       m_fragId = fid;
       m_state = SFH_NOT_STARTED;
       m_rangePtrI = RNIL;
+      m_readBackup = readBackup;
       reset_ranges();
     }
 
     Uint32 m_magic;
     Uint32 m_treeNodePtrI;
     Uint16 m_fragId;
-    Uint16 m_state;
+    Uint8 m_state;
+    Uint8 m_readBackup;
     Uint32 m_ref;
 
     void reset_ranges() {
@@ -1017,6 +1020,9 @@ public:
     PatternStore::Head m_keyPattern;
     PatternStore::Head m_attrParamPattern;
 
+    // Memory Arena with lifetime limited to current result batch / node
+    ArenaHead m_batchArena;
+
     /**
      * Rows buffered by this node
      */
@@ -1102,7 +1108,7 @@ public:
 
       RS_ABORTED    = 0x2008, // Aborted and waiting for SCAN_NEXTREQ
       RS_END = 0
-    };  //struct Request
+    };
 
     Request() {}
     Request(const ArenaHead & arena) : m_arena(arena) {}
@@ -1156,7 +1162,7 @@ public:
       Uint32 nextPool;
     };
     Uint32 prevHash;
-  };
+  }; //struct Request
 
 private:
   /**
@@ -1320,9 +1326,9 @@ private:
   Uint32 build(Build_context&,Ptr<Request>,SectionReader&,SectionReader&);
   Uint32 initRowBuffers(Ptr<Request>);
   void buildExecPlan(Ptr<Request>, Ptr<TreeNode> node, Ptr<TreeNode> next);
-  void checkPrepareComplete(Signal*, Ptr<Request>, Uint32 cnt);
-  void start(Signal*, Ptr<Request>);
-  void checkBatchComplete(Signal*, Ptr<Request>, Uint32 cnt);
+  void prepare(Signal*, Ptr<Request>);
+  void checkPrepareComplete(Signal*, Ptr<Request>);
+  void checkBatchComplete(Signal*, Ptr<Request>);
   void batchComplete(Signal*, Ptr<Request>);
   void prepareNextBatch(Signal*, Ptr<Request>);
   void sendConf(Signal*, Ptr<Request>, bool is_complete);
@@ -1447,8 +1453,7 @@ private:
   void lookup_start(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_resume(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_send(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void lookup_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
-			     const RowPtr&);
+  bool lookup_countSignal(const Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_execLQHKEYREF(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_execLQHKEYCONF(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_stop_branch(Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 err);
@@ -1482,8 +1487,7 @@ private:
                         const QueryNode*, const QueryNodeParameters*);
   void scanFrag_start(Signal*, Ptr<Request>,Ptr<TreeNode>);
   void scanFrag_send(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void scanFrag_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
-			       const RowPtr &);
+  bool scanFrag_countSignal(const Signal*, Ptr<Request>, Ptr<TreeNode>);
   void scanFrag_execSCAN_FRAGREF(Signal*, Ptr<Request>, Ptr<TreeNode>, Ptr<ScanFragHandle>);
   void scanFrag_execSCAN_FRAGCONF(Signal*, Ptr<Request>, Ptr<TreeNode>, Ptr<ScanFragHandle>);
   void scanFrag_execSCAN_NEXTREQ(Signal*, Ptr<Request>,Ptr<TreeNode>);
@@ -1504,8 +1508,7 @@ private:
                         DABuffer tree, Uint32 treeBits,
                         DABuffer param, Uint32 paramBits);
   void scanIndex_prepare(Signal*, Ptr<Request>, Ptr<TreeNode>);
-  void scanIndex_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
-                                const RowPtr &);
+  bool scanIndex_countSignal(const Signal*, Ptr<Request>, Ptr<TreeNode>);
   void scanIndex_execSCAN_FRAGREF(Signal*, Ptr<Request>, Ptr<TreeNode>, Ptr<ScanFragHandle>);
   void scanIndex_execSCAN_FRAGCONF(Signal*, Ptr<Request>, Ptr<TreeNode>, Ptr<ScanFragHandle>);
   void scanIndex_parent_row(Signal*,Ptr<Request>,Ptr<TreeNode>, const RowPtr&);
