@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -923,7 +923,8 @@ void Trix::execSUB_SYNC_REF(Signal* signal)
     DBUG_VOID_RETURN;
   }
   subRecPtr.p = subRec;
-  buildFailed(signal, subRecPtr, BuildIndxRef::InternalError);
+  buildFailed(signal, subRecPtr,
+              (BuildIndxRef::ErrorCode)subSyncRef->errorCode);
   DBUG_VOID_RETURN;
 }
 
@@ -1010,6 +1011,8 @@ void Trix::setupSubscription(Signal* signal, SubscriptionRecPtr subRecPtr)
 		     subRecPtr.i, subCreateReq->subscriptionId,
 		     subCreateReq->subscriptionKey));
 
+  D("SUB_CREATE_REQ tableId: " << subRec->sourceTableId);
+
   sendSignal(SUMA_REF, GSN_SUB_CREATE_REQ, 
 	     signal, SubCreateReq::SignalLength, JBB);
 
@@ -1039,7 +1042,7 @@ void Trix::startTableScan(Signal* signal, SubscriptionRecPtr subRecPtr)
   }
 
   // Merge index and key column segments
-  struct LinearSectionPtr orderPtr[3];
+  LinearSectionPtr orderPtr[3];
   Uint32 noOfSections;
   orderPtr[0].p = attributeList;
   orderPtr[0].sz = cnt;
@@ -1076,13 +1079,13 @@ void Trix::startTableScan(Signal* signal, SubscriptionRecPtr subRecPtr)
   {
     jam();
     subSyncReq->requestInfo |= SubSyncReq::LM_Exclusive;
-    subSyncReq->requestInfo |= SubSyncReq::Reorg;
+    subSyncReq->requestInfo |= SubSyncReq::ReorgDelete;
   }
   else if (subRec->requestType == STAT_CLEAN)
   {
     jam();
     StatOp& stat = statOpGetPtr(subRecPtr.p->m_statPtrI);
-    StatOp::Clean clean = stat.m_clean;
+    StatOp::Clean& clean = stat.m_clean;
     orderPtr[1].p = clean.m_bound;
     orderPtr[1].sz = clean.m_boundSize;
     noOfSections = 2;
@@ -1105,6 +1108,10 @@ void Trix::startTableScan(Signal* signal, SubscriptionRecPtr subRecPtr)
 		     subRecPtr.i, subSyncReq->subscriptionId,
 		     subSyncReq->subscriptionKey));
 
+  D("SUB_SYNC_REQ fragId: " << subRec->fragId <<
+    " fragCount: " << subRec->fragCount <<
+    " requestInfo: " << hex << subSyncReq->requestInfo);
+
   sendSignal(SUMA_REF, GSN_SUB_SYNC_REQ,
 	     signal, SubSyncReq::SignalLength, JBB, orderPtr, noOfSections);
 }
@@ -1610,6 +1617,12 @@ Trix::execCOPY_DATA_IMPL_REQ(Signal* signal)
     subRec->noOfKeyColumns = ptr.sz;
   }
 
+  D("COPY_DATA_IMPL_REQ srctableId: " << subRec->sourceTableId <<
+    " targetTableId: " << subRec->targetTableId <<
+    " fragCount: " << subRec->fragCount <<
+    " requestType: " << subRec->requestType <<
+    " flags: " << hex << subRec->m_flags);
+
   releaseSections(handle);
   {
     UtilPrepareReq * utilPrepareReq =
@@ -1632,7 +1645,10 @@ Trix::execCOPY_DATA_IMPL_REQ(Signal* signal)
     {
       w.add(UtilPrepareReq::OperationType, UtilPrepareReq::Delete);
     }
-    w.add(UtilPrepareReq::ScanTakeOverInd, 1);
+    if (!(req->requestInfo & CopyDataReq::NoScanTakeOver))
+    {
+      w.add(UtilPrepareReq::ScanTakeOverInd, 1);
+    }
     w.add(UtilPrepareReq::ReorgInd, 1);
     w.add(UtilPrepareReq::TableId, subRec->targetTableId);
 
@@ -2651,30 +2667,32 @@ Trix::statCleanPrepare(Signal* signal, StatOp& stat)
   clean.m_bound[3] = TuxBoundInfo::BoundEQ;
   clean.m_bound[4] = AttributeHeader(1, 4).m_value;
   clean.m_bound[5] = data.m_indexVersion;
+  Uint32 boundCount;
   switch (stat.m_requestType) {
   case IndexStatReq::RT_CLEAN_NEW:
     D("statCleanPrepare delete sample versions > " << data.m_sampleVersion);
     clean.m_bound[6] = TuxBoundInfo::BoundLT;
     clean.m_bound[7] = AttributeHeader(2, 4).m_value;
     clean.m_bound[8] = data.m_sampleVersion;
-    clean.m_boundCount = 3;
+    boundCount = 3;
     break;
   case IndexStatReq::RT_CLEAN_OLD:
     D("statCleanPrepare delete sample versions < " << data.m_sampleVersion);
     clean.m_bound[6] = TuxBoundInfo::BoundGT;
     clean.m_bound[7] = AttributeHeader(2, 4).m_value;
     clean.m_bound[8] = data.m_sampleVersion;
-    clean.m_boundCount = 3;
+    boundCount = 3;
     break;
   case IndexStatReq::RT_CLEAN_ALL:
     D("statCleanPrepare delete all sample versions");
-    clean.m_boundCount = 2;
+    boundCount = 2;
     break;
   default:
+    boundCount = 0; /* Silence compiler warning */
     ndbrequire(false);
     break;
   }
-  clean.m_boundSize = 3 * clean.m_boundCount;
+  clean.m_boundSize = 3 * boundCount;
 
   // TRIX traps the CONF
   send.m_sysTable = &g_statMetaSample;
@@ -2710,7 +2728,8 @@ Trix::statCleanExecute(Signal* signal, StatOp& stat)
   ndbrequire(ah[3].getAttributeId() == 3 && kz != 0);
 
   // AFTER_VALUES
-  const Uint32 avmax = 3 + MAX_INDEX_STAT_KEY_SIZE;
+  // avmax = other pk attributes + length + max index stat key size
+  const Uint32 avmax = 3 + 1 + MAX_INDEX_STAT_KEY_SIZE;
   Uint32 av[avmax];
   SegmentedSectionPtr ptr1;
   handle.getSection(ptr1, SubTableData::AFTER_VALUES);
@@ -2885,7 +2904,8 @@ Trix::statScanExecute(Signal* signal, StatOp& stat)
   ndbrequire(kz != 0 && vz != 0);
 
   // AFTER_VALUES
-  const Uint32 avmax = MAX_INDEX_STAT_KEY_SIZE + MAX_INDEX_STAT_VALUE_SIZE;
+  // avmax = length + max key size + length + max value size
+  const Uint32 avmax = 2 + MAX_INDEX_STAT_KEY_SIZE + MAX_INDEX_STAT_VALUE_SIZE;
   Uint32 av[avmax];
   SegmentedSectionPtr ptr1;
   handle.getSection(ptr1, SubTableData::AFTER_VALUES);
@@ -2976,7 +2996,7 @@ Trix::statScanEnd(Signal* signal, StatOp& stat)
    * we prefer DbtuxProxy to avoid introducing MT-LQH into TRIX.
    */
 
-#if trix_index_stat_rep_to_tux_instance
+#ifdef trix_index_stat_rep_to_tux_instance
   Uint32 instanceKey = getInstanceKey(req->indexId, req->fragId);
   BlockReference tuxRef = numberToRef(DBTUX, instanceKey, getOwnNodeId());
 #else
