@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2010, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,63 +24,86 @@
 
 #include "trp_client.hpp"
 #include "TransporterFacade.hpp"
-#include <NdbMem.h>
 
 trp_client::trp_client()
-  : m_blockNo(~Uint32(0)), m_facade(0)
+  : m_blockNo(~Uint32(0)),
+    m_facade(NULL),
+    m_locked_for_poll(false),
+    m_mutex(NULL),
+    m_poll(),
+    m_enabled_nodes_mask(),
+    m_send_nodes_cnt(0),
+    m_send_nodes_mask(),
+    m_send_buffers(NULL)
 {
   m_mutex = NdbMutex_Create();
-
-  m_send_nodes_cnt = 0;
   m_send_buffers = new TFBuffer[MAX_NODES];
 }
 
 trp_client::~trp_client()
 {
-  /**
-   * require that trp_client user
-   *  doesnt destroy object when holding any locks
-   */
-  m_poll.assert_destroy();
-
   close();
-  NdbCondition_Destroy(m_poll.m_condition);
-  m_poll.m_condition = NULL;
   NdbMutex_Destroy(m_mutex);
-  m_mutex = NULL;
 
+  m_mutex = NULL;
   assert(m_send_nodes_cnt == 0);
+  assert(m_locked_for_poll == false);
   delete [] m_send_buffers;
 }
 
+trp_client::PollQueue::PollQueue()
+  : m_waiting(PQ_IDLE),
+    m_locked(false),
+    m_poll_owner(false),
+    m_poll_queue(false),
+    m_prev(NULL),
+    m_next(NULL),
+    m_condition(NULL)
+{
+  m_condition = NdbCondition_Create();
+}
+
+trp_client::PollQueue::~PollQueue()
+{
+  /**
+   * Require that trp_client user
+   * doesnt destroy object when holding any locks.
+   */
+  if (unlikely(
+      m_waiting != PQ_IDLE ||
+      m_locked == true ||
+      m_poll_owner == true ||
+      m_poll_queue == true ||
+      m_next != 0 ||
+      m_prev != 0))
+  {
+    ndbout << "ERR: ::~PollQueue: Deleting trp_clnt in use: waiting"
+           << m_waiting
+	   << " locked  " << m_locked
+	   << " poll_owner " << m_poll_owner
+	   << " poll_queue " << m_poll_queue
+	   << " next " << m_next
+	   << " prev " << m_prev
+           << endl;
+    require(false);
+  }
+  NdbCondition_Destroy(m_condition);
+  m_condition = NULL;
+}
+
 Uint32
-trp_client::open(TransporterFacade* tf, int blockNo,
-                 bool receive_thread)
+trp_client::open(TransporterFacade* tf, int blockNo)
 {
   Uint32 res = 0;
-  assert(m_facade == 0);
-  if (m_facade == 0)
+  assert(m_enabled_nodes_mask.isclear());
+  assert(m_facade == NULL);
+  if (m_facade == NULL)
   {
     m_facade = tf;
-    /**
-      For user threads we only store up to 16 threads before waking
-      them up, for receiver threads we store up to 256 threads before
-      waking them up.
-    */
-    if (receive_thread)
-    {
-      m_poll.m_lock_array_size = 256;
-    }
-    else
-    {
-      m_poll.m_lock_array_size = 16;
-    }
-    m_poll.m_locked_clients = (trp_client**)
-      NdbMem_Allocate(sizeof(trp_client**) * m_poll.m_lock_array_size);
-    if (m_poll.m_locked_clients == NULL)
-    {
-      return 0;
-    }
+
+    /* Initially allowed to communicate with itself */
+    m_enabled_nodes_mask.set(m_facade->theOwnId);
+
     res = tf->open_clnt(this, blockNo);
     if (res != 0)
     {
@@ -88,9 +111,7 @@ trp_client::open(TransporterFacade* tf, int blockNo,
     }
     else
     {
-      NdbMem_Free(m_poll.m_locked_clients);
-      m_poll.m_locked_clients = NULL;
-      m_facade = 0;
+      m_facade = NULL;
     }
   }
   return res;
@@ -109,28 +130,87 @@ trp_client::close()
   {
     m_facade->close_clnt(this);
 
-    m_facade = 0;
+    m_facade = NULL;
     m_blockNo = ~Uint32(0);
-    if (m_poll.m_locked_clients)
-    {
-      NdbMem_Free(m_poll.m_locked_clients);
-      m_poll.m_locked_clients = NULL;
-    }
   }
+  m_enabled_nodes_mask.clear();
+}
+
+/**
+ * Initial setup of the nodes having their send buffers enabled.
+ * Callback from TransporterFacade::open_clnt' called above.
+ *
+ * Protected by having the 'm_mutex' locked
+ */
+void
+trp_client::set_enabled_send(const NodeBitmask &nodes)
+{
+  assert(NdbMutex_Trylock(m_mutex) != 0);
+  m_enabled_nodes_mask.assign(nodes);
+}
+
+/**
+ * Enable/disable of trp_client sending.
+ *
+ * Protected by either locking the m_mutex, or being 'parked'
+ * in the inactive poll queue (is_locked_for_poll())
+ *
+ * Called from TransporterFacade when TF enable/disable
+ * its own 'global' send buffers.
+ *
+ * When disabling the send_buffers any buffered data is also
+ * discarded. The TransportRegistry 'protocol' also requires
+ * isSendEnabled() to be checked before attempting to allocate
+ * send buffers (getWritePtr() / updateWritePtr).
+ *
+ * No WritePtr allocation should be attempted from a disabled send buffer.
+ * Furthermore, finding pending send data to flush for a disabled send
+ * buffer will indicate a concurrency control problem. (Asserted)
+ */
+void
+trp_client::enable_send(NodeId node)
+{
+  assert(m_poll.m_locked || NdbMutex_Trylock(m_mutex) != 0);
+  m_enabled_nodes_mask.set(node);
 }
 
 void
-trp_client::start_poll()
+trp_client::disable_send(NodeId node)
+{
+  assert(m_poll.m_locked || NdbMutex_Trylock(m_mutex) != 0);
+  if (m_send_nodes_mask.get(node))
+  {
+    // Discard any buffered data to disabled node.
+    TFBuffer* b = m_send_buffers + node;
+    TFBufferGuard g0(* b);
+    m_facade->m_send_buffer.release_list(b->m_head);
+    b->clear();
+  }
+  m_enabled_nodes_mask.clear(node);
+}
+
+/**
+ * The client has to 'poll' the transporter in order to receive
+ * its result. The call to ::do_poll() should be encapsulate with
+ * a ::prepare_poll() - ::complete_poll() pair.
+ */
+  void
+trp_client::prepare_poll()
 {
   NdbMutex_Lock(m_mutex);
   assert(m_poll.m_locked == false);
+  assert(m_poll.m_poll_queue == false);
+  assert(m_poll.m_waiting == trp_client::PollQueue::PQ_IDLE);
+  assert(has_unflushed_sends() == false); //Flushed prior to poll-wait
   m_poll.m_locked = true;
-  m_facade->start_poll(this);
 }
 
 void
 trp_client::do_poll(Uint32 to)
 {
+  assert(m_poll.m_locked == true);
+  assert(m_poll.m_poll_queue == false);
+  assert(m_poll.m_waiting == trp_client::PollQueue::PQ_IDLE);
   m_facade->do_poll(this, to);
 }
 
@@ -138,7 +218,14 @@ void
 trp_client::complete_poll()
 {
   assert(m_poll.m_locked == true);
-  m_facade->complete_poll(this);
+  assert(m_poll.m_poll_queue == false);
+  assert(m_poll.m_waiting == trp_client::PollQueue::PQ_IDLE);
+  /**
+   * Ensure any signals sent by receiver/poll owner has been
+   * flushed to the global Transporter buffers.
+   * The send thread will eventually send the transporter buffers.
+   */
+  assert(has_unflushed_sends() == false);
   m_poll.m_locked = false;
   NdbMutex_Unlock(m_mutex);
 }
@@ -161,10 +248,12 @@ trp_client::do_forceSend(int val)
   {
     for (Uint32 i = 0; i < m_send_nodes_cnt; i++)
     {
-      Uint32 n = m_send_nodes_list[i];
-      TFBuffer* b = m_send_buffers + n;
+      Uint32 node = m_send_nodes_list[i];
+      assert(m_send_nodes_mask.get(node));
+      assert(m_enabled_nodes_mask.get(node));
+      TFBuffer* b = m_send_buffers + node;
       TFBufferGuard g0(* b);
-      m_facade->flush_and_send_buffer(n, b);
+      m_facade->flush_and_send_buffer(node, b);
       b->clear();
     }
     m_send_nodes_cnt = 0;
@@ -174,12 +263,54 @@ trp_client::do_forceSend(int val)
   return 0;
 }
 
+/**
+ * Flush client privat send buffers to the global
+ * TransporterFacade list of sendbuffers.
+ *
+ * Need to be called with the 'm_mutex' held
+ */
+void
+trp_client::flush_send_buffers()
+{
+  assert(m_poll.m_locked);
+  const Uint32 cnt = m_send_nodes_cnt;
+  for (Uint32 i = 0; i<cnt; i++)
+  {
+    const Uint32 node = m_send_nodes_list[i];
+    assert(m_send_nodes_mask.get(node));
+    assert(m_enabled_nodes_mask.get(node));
+    TFBuffer* b = m_send_buffers + node;
+    TFBufferGuard g0(* b);
+    m_facade->flush_send_buffer(node, b);
+    b->clear();
+  }
+  m_send_nodes_cnt = 0;
+  m_send_nodes_mask.clear();
+}
+
+/**
+ * The 'safe' sendSignal() methods has to be used instead of the
+ * other sendSignal methods when a reply signal has to be
+ * sent by the client getting a signal 'delivered'.
+ *
+ * See 'is_poll_owner_thread()'-comments for more details.
+ */
 int
 trp_client::safe_noflush_sendSignal(const NdbApiSignal* signal, Uint32 nodeId)
 {
+  // This thread must be the poll owner
+  assert(m_facade->is_poll_owner_thread());
   return m_facade->m_poll_owner->raw_sendSignal(signal, nodeId);
 }
 
+int trp_client::safe_noflush_sendSignal(const NdbApiSignal* signal, Uint32 nodeId,
+                                        const LinearSectionPtr ptr[3], Uint32 secs)
+{
+  // This thread must be the poll owner
+  assert(m_facade->is_poll_owner_thread());
+  return m_facade->m_poll_owner->raw_sendSignal(signal, nodeId, ptr, secs);
+}
+
 int
 trp_client::safe_sendSignal(const NdbApiSignal* signal, Uint32 nodeId)
 {
@@ -191,10 +322,40 @@ trp_client::safe_sendSignal(const NdbApiSignal* signal, Uint32 nodeId)
   return res;
 }
 
+int
+trp_client::safe_sendSignal(const NdbApiSignal* signal, Uint32 nodeId,
+                            const LinearSectionPtr ptr[3], Uint32 secs)
+{
+  int res;
+  if ((res = safe_noflush_sendSignal(signal, nodeId, ptr, secs)) != -1)
+  {
+    m_facade->m_poll_owner->flush_send_buffers();
+  }
+  return res;
+}
+
+/**
+ * ::isSendEnabled() and get-/updateWritePtr()
+ *
+ * We assume (and assert) that TransporterRegistry::prepareSend()
+ * check whether a node is 'isSendEnabled()' before allocating send buffer
+ * for a node send by calling getWritePtr() - updateWritePtr().
+ *
+ * Requires the 'm_mutex' to be held prior to calling these functions
+ */
+bool
+trp_client::isSendEnabled(NodeId node) const
+{
+  assert(m_poll.m_locked);
+  return m_enabled_nodes_mask.get(node);
+}
+
 Uint32 *
 trp_client::getWritePtr(NodeId node, Uint32 lenBytes, Uint32 prio,
                         Uint32 max_use)
 {
+  assert(isSendEnabled(node));
+  
   TFBuffer* b = m_send_buffers+node;
   TFBufferGuard g0(* b);
   bool found = m_send_nodes_mask.get(node);
@@ -266,23 +427,33 @@ trp_client::updateWritePtr(NodeId node, Uint32 lenBytes, Uint32 prio)
   return b->m_bytes_in_buffer;
 }
 
+/**
+ * This is the implementation used by the NDB API. I update the
+ * current send buffer size every time a thread gets the send mutex and
+ * links their buffers to the common pool of buffers. I recalculate the
+ * buffer size also every time a send to the node has been completed.
+ *
+ * The values we read here are read unprotected, the idea is that the
+ * value reported from here should only used for guidance. So it should
+ * only implement throttling, it should not completely stop send activities,
+ * merely delay it. So the harm in getting an inconsistent view of data
+ * should not be high. Also we expect measures of slowing down to occur
+ * at a fairly early stage, so not close to when the buffers are filling up.
+ */
 void
-trp_client::flush_send_buffers()
+trp_client::getSendBufferLevel(NodeId node, SB_LevelType &level)
 {
-  assert(m_poll.m_locked);
-  Uint32 cnt = m_send_nodes_cnt;
-  for (Uint32 i = 0; i<cnt; i++)
-  {
-    Uint32 node = m_send_nodes_list[i];
-    assert(m_send_nodes_mask.get(node));
-    TFBuffer* b = m_send_buffers + node;
-    TFBufferGuard g0(* b);
-    m_facade->flush_send_buffer(node, b);
-    b->clear();
-  }
-
-  m_send_nodes_cnt = 0;
-  m_send_nodes_mask.clear();
+  Uint32 current_send_buffer_size = m_facade->get_current_send_buffer_size(node);
+  Uint64 tot_send_buffer_size =
+    m_facade->m_send_buffer.get_total_send_buffer_size();
+  Uint64 tot_used_send_buffer_size =
+    m_facade->m_send_buffer.get_total_used_send_buffer_size();
+  calculate_send_buffer_level(current_send_buffer_size,
+                              tot_send_buffer_size,
+                              tot_used_send_buffer_size,
+                              0,
+                              level);
+  return;
 }
 
 bool
@@ -298,7 +469,7 @@ PollGuard::PollGuard(NdbImpl& impl)
 {
   m_clnt = &impl;
   m_waiter= &impl.theWaiter;
-  m_clnt->start_poll();
+  m_clnt->prepare_poll();
   m_complete_poll_called = false;
 }
 
@@ -335,27 +506,35 @@ int PollGuard::wait_scan(int wait_time, Uint32 nodeId, bool forceSend)
   return wait_for_input_in_loop(wait_time, forceSend);
 }
 
-int PollGuard::wait_for_input_in_loop(int wait_time, bool forceSend)
+int PollGuard::wait_for_input_in_loop(int max_wait_ms, bool forceSend)
 {
   int ret_val;
   m_clnt->do_forceSend(forceSend ? 1 : 0);
 
-  NDB_TICKS curr_ticks = NdbTick_getCurrentTicks();
-  /* Use nanosecond to calculate when wait_time has expired. */
-  Int64 remain_wait_nano = ((Int64)wait_time) * 1000000;
-  const int maxsleep = (wait_time == -1 || wait_time > 10) ? 10 : wait_time;
+  const NDB_TICKS start = NdbTick_getCurrentTicks();
+  int remain_wait_ms = max_wait_ms;
 #ifdef VM_TRACE
   const bool verbose = (m_waiter->get_state() != WAIT_EVENT);
 #endif
   do
   {
+    int maxsleep = (max_wait_ms == -1) ? 60*1000 : remain_wait_ms;
+    DBUG_EXECUTE_IF("ndb_simulate_nodefail", {
+      if (maxsleep > 10)
+        maxsleep = 10;
+    });
     wait_for_input(maxsleep);
-    const NDB_TICKS start_ticks = curr_ticks;
-    curr_ticks = NdbTick_getCurrentTicks();
-    const Uint64 waited_nano = NdbTick_Elapsed(start_ticks,curr_ticks).nanoSec();
-    m_clnt->recordWaitTimeNanos(waited_nano);
+
+    const NDB_TICKS now = NdbTick_getCurrentTicks();
+    m_clnt->recordWaitTimeNanos(NdbTick_Elapsed(start,now).nanoSec());
     Uint32 state= m_waiter->get_state();
-    if (state == NO_WAIT)
+
+    DBUG_EXECUTE_IF("ndb_simulate_nodefail", {
+      DBUG_PRINT("info", ("Simulating node failure while waiting for response"));
+      state = WAIT_NODE_FAILURE;
+    });
+
+    if (likely(state == NO_WAIT))
     {
       return 0;
     }
@@ -365,15 +544,17 @@ int PollGuard::wait_for_input_in_loop(int wait_time, bool forceSend)
       m_waiter->set_state(NO_WAIT);
       break;
     }
-    if (wait_time == -1)
+    if (max_wait_ms == -1)
     {
 #ifdef NOT_USED
       ndbout << "Waited WAITFOR_RESPONSE_TIMEOUT, continuing wait" << endl;
 #endif
       continue;
     }
-    remain_wait_nano -= waited_nano;
-    if (remain_wait_nano <= 0)
+    remain_wait_ms = max_wait_ms -
+      (int)NdbTick_Elapsed(start,now).milliSec();
+
+    if (remain_wait_ms <= 0)
     {
 #ifdef VM_TRACE
       if (verbose)
@@ -386,10 +567,9 @@ int PollGuard::wait_for_input_in_loop(int wait_time, bool forceSend)
       break;
     }
     /**
-     * Ensure any signals sent by receivers are sent by send thread
-     * eventually by flushing buffers to global area.
+     * Ensure no reply-signals sent by receivers remains unflushed.
      */
-    m_clnt->flush_send_buffers();
+    assert(m_clnt->has_unflushed_sends() == false);
   } while (1);
 #ifdef VM_TRACE
   if (verbose)
