@@ -1,18 +1,24 @@
-/* Copyright (c) 2008, 2014 Oracle and/or its affiliates.  All rights reserved
+/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
 
 #include <NDBT.hpp>
 #include <NDBT_Test.hpp>
@@ -31,6 +37,10 @@
     trans->close();                                   \
     return NDBT_FAILED; }
 
+#define CHECK(v) if (!(v)) {                      \
+    ndbout << "Error at line " << __LINE__ <<         \
+      endl;                                           \
+    return NDBT_FAILED; }
 
 /* Setup memory as a long Varchar with 2 bytes of
  * length information
@@ -1054,7 +1064,7 @@ int create100Tables(NDBT_Context* ctx, NDBT_Step* step)
   for (Uint32 t=0; t < 100; t++)
   {
     char tabnameBuff[10];
-    snprintf(tabnameBuff, sizeof(tabnameBuff), "TAB%u", t);
+    BaseString::snprintf(tabnameBuff, sizeof(tabnameBuff), "TAB%u", t);
     
     NdbDictionary::Table tab;
     tab.setName(tabnameBuff);
@@ -1095,7 +1105,7 @@ int drop100Tables(NDBT_Context* ctx, NDBT_Step* step)
   for (Uint32 t=0; t < 100; t++)
   {
     char tabnameBuff[10];
-    snprintf(tabnameBuff, sizeof(tabnameBuff), "TAB%u", t);
+    BaseString::snprintf(tabnameBuff, sizeof(tabnameBuff), "TAB%u", t);
     
     if (pNdb->getDictionary()->dropTable(tabnameBuff) != 0)
     {
@@ -1124,7 +1134,7 @@ int dropTable(NDBT_Context* ctx, NDBT_Step* step, Uint32 num)
     return NDBT_OK;
     
   char tabnameBuff[10];
-  snprintf(tabnameBuff, sizeof(tabnameBuff), "TAB%u", num);
+  BaseString::snprintf(tabnameBuff, sizeof(tabnameBuff), "TAB%u", num);
   
   if (pNdb->getDictionary()->dropTable(tabnameBuff) != 0)
   {
@@ -1365,6 +1375,49 @@ int testSlowDihFileWrites(NDBT_Context* ctx, NDBT_Step* step)
   return NDBT_OK;
 }
 
+int testNdbfsBulkOpen(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbRestarter restarter;
+
+  g_err << "Getting all nodes to create + open a number of files in parallel"
+        << endl;
+  int dumpArg = 667;
+  CHECK(restarter.dumpStateAllNodes(&dumpArg, 1) == 0);
+
+  ndbout_c("Giving time for the open to complete");
+  NdbSleep_MilliSleep(30*1000);
+
+  ndbout_c("Crash DB nodes that have not completed opening files");
+  dumpArg = 668;
+  CHECK(restarter.dumpStateAllNodes(&dumpArg, 1) == 0);
+
+  g_err << "Checking any data node crashed" << endl;
+  uint num_nodes = restarter.getNumDbNodes();
+  int *dead_nodes = new int[num_nodes];
+  for (uint i = 0; i < num_nodes; ++i)
+  {
+    dead_nodes[i] = 0;
+  }
+  int dead_node = restarter.checkClusterAlive(dead_nodes, num_nodes);
+  if (dead_node != 0)
+  {
+    g_err << "Data node " << dead_node << " crashed" << endl;
+  }
+  CHECK(dead_node == 0);
+
+  g_err << "Restarting nodes to get rid of error insertion effects"
+        << endl;
+  // restartAll(initial=true) doesn't remove CMVMI either
+  CHECK(restarter.restartAll() == 0);
+  const int timeout = 300;
+  CHECK(restarter.waitClusterStarted(timeout) == 0);
+  Ndb* pNdb = GETNDB(step);
+  CHECK(pNdb->waitUntilReady(timeout) == 0);
+  CHK_NDB_READY(pNdb);
+
+  return NDBT_OK;
+}
+
 
 NDBT_TESTSUITE(testLimits);
 
@@ -1395,6 +1448,11 @@ TESTCASE("SlowDihFileWrites",
   STEP(testSlowDihFileWrites);
   FINALIZER(drop100Tables);
 }
+TESTCASE("NdbfsBulkOpen",
+         "Test behaviour of NdbFs bulk file open")
+{
+  INITIALIZER(testNdbfsBulkOpen);
+}
 
 NDBT_TESTSUITE_END(testLimits);
 
